import camelcase from "camelcase";
import Embed from "./Embed.html";

const embdAtts = [
  "compositionid",
  "id",
  "ref",
  "wref",
  "widget_id",
  "widgetid",
  "widget",
  "theme",
  "inline",
  "autoscale",
  "resize",
  "width",
  "height",
  "adaptive",
  "align",
  "branding",
];

function attributesToObject(attributes) {
  return Array.prototype.slice.call(attributes).reduce((acc, attribute) => {
    acc[attribute.name] = attribute.value;
    return acc;
  }, {});
}

function getWidgetTags(document) {
  return Array.prototype.slice
    .call(document.querySelectorAll("*"))
    .filter((el) => el.tagName.indexOf("WIDGETIC-") === 0 || el.tagName === "WIDGETIC" || 
                    Object.values(el.attributes).map((el) => el.nodeName).includes('widgetic'))
    .filter((el) => el.__wdgtcparsed != true);
}

function extractConfig(el) {
  const embedOptions = embdAtts
    .map((a) => {
      const value = el.getAttribute(a);
      if (value === null) return undefined;
      return { [a]: value };
    })
    .filter((i) => i !== undefined)
    .reduce((i, acc) => Object.assign(i, acc));
  return {
    target: el,
    data: Object.assign(embedOptions, {
      theme: extractTheme(el),
      content: extractContent(el),
    }),
  };
}

function extractTheme(el) {
  // return el.dataset;
  const attrs = attributesToObject(el.attributes);
  let theme = {};
  for (var attr in attrs) {
    if (embdAtts.includes(attr)) continue;
    theme[camelcase(attr.replace("data-", ""))] = attrs[attr];
  }

  if (attrs["theme"]) theme.id = attrs["theme"];

  return theme;
}

function extractContent(el) {
  return Array.prototype.slice
    .call(el.children)
    .map((child) => ({
      attrs: attributesToObject(child.attributes),
      content: child.innerHTML,
    }))
    .map(({ attrs, content }, i) => {
      const item = {
        id: i + 1,
        order: i + 1,
        metadata: { service: "declarative" },
      };

      for (var attr in attrs) {
        item.metadata[camelcase(attr.replace("data-", ""))] = attrs[attr];
      }

      if (content) item.metadata.content = content.trim();
      return item;
    });
}

function createEmbed(config) {
  config.target.__wdgtcparsed = true;
  config.target.innerHTML = "";
  new Embed(config);
}

export const extractDeclarativeEmbeds = function (doc = document) {
  return getWidgetTags(doc).map(extractConfig);
};

export const parseDeclarativeEmbeds = function () {
  extractDeclarativeEmbeds().map(createEmbed);
};
