import test from 'ava';
import { extractDeclarativeEmbeds } from './declarative';
import jsdom from 'jsdom';
const { JSDOM } = jsdom;

function dom(html) {
  return new JSDOM('<!DOCTYPE html>' + html).window.document;
}

test('parse composition', t => {
  const embeds = extractDeclarativeEmbeds(dom`
    <widgetic-embed id="123"></widgetic-embed>
  `);
  t.deepEqual([{ id: '123', theme: {}, content: [] }], embeds.map(i => i.data));
});

test('parse widget', t => {
  const embeds = extractDeclarativeEmbeds(dom`
    <widgetic-embed widget="123"></widgetic-embed>
  `);
  t.deepEqual(
    [{ widget: '123', theme: {}, content: [] }],
    embeds.map(i => i.data)
  );
});

test('parse mutliple embeds', t => {
  const embeds = extractDeclarativeEmbeds(dom`
    <widgetic-embed id="123"></widgetic-embed>
    <widgetic-embed widget="123"></widgetic-embed>
  `);
  t.deepEqual(
    [
      { id: '123', theme: {}, content: [] },
      { widget: '123', theme: {}, content: [] },
    ],
    embeds.map(i => i.data)
  );
});

test('configure resize options', t => {
  const embeds = extractDeclarativeEmbeds(dom`
    <widgetic-embed id="123" autoscale="on" resize="fixed" width="200" height="80vh" adaptive="414" align="center"></widgetic-embed>
  `);
  t.deepEqual(
    [
      {
        id: '123',
        autoscale: 'on',
        resize: 'fixed',
        height: '80vh',
        width: '200',
        adaptive: '414',
        align: 'center',
        theme: {},
        content: [],
      },
    ],
    embeds.map(i => i.data)
  );
});

test('specify theme', t => {
  const embeds = extractDeclarativeEmbeds(dom`
    <widgetic-embed widget="123" theme="p2"></widgetic-embed>
  `);
  t.deepEqual(
    [{ widget: '123', theme: { id: 'p2' }, content: [] }],
    embeds.map(i => i.data)
  );
});

test('override theme attributes', t => {
  const embeds = extractDeclarativeEmbeds(dom`
    <widgetic-embed widget="123" backgroundcolor="red" item-highlight-color="blue" data-width="200"></widgetic-embed>
  `);
  t.deepEqual(
    [
      {
        widget: '123',
        theme: {
          backgroundcolor: 'red',
          itemHighlightColor: 'blue',
          width: '200',
        },
        content: [],
      },
    ],
    embeds.map(i => i.data)
  );
});

test('specify theme and override attributes', t => {
  const embeds = extractDeclarativeEmbeds(dom`
    <widgetic-embed widget="123" theme="p2" backgroundcolor="red"></widgetic-embed>
  `);
  t.deepEqual(
    [
      {
        widget: '123',
        theme: { backgroundcolor: 'red', id: 'p2' },
        content: [],
      },
    ],
    embeds.map(i => i.data)
  );
});

test('configure content', t => {
  const embeds = extractDeclarativeEmbeds(dom`
      <widgetic-embed widget="123">
        <img src="#" alt="Description" data-width="200" border-color="red"/>
        <some-tag wtf="#"></some-tag>
        <section color="red">
          <h1>Hello world</h1>
          Hi there! How are <super>you</super> doing?
        </section>
      </widgetic-embed>
    `);
  t.deepEqual(
    [
      {
        widget: '123',
        theme: {},
        content: [
          {
            id: 1,
            order: 1,
            metadata: {
              service: 'declarative',
              src: '#',
              alt: 'Description',
              width: '200',
              borderColor: 'red',
            },
          },
          {
            id: 2,
            order: 2,
            metadata: { service: 'declarative', wtf: '#' },
          },
          {
            id: 3,
            order: 3,
            metadata: {
              service: 'declarative',
              color: 'red',
              content:
                '<h1>Hello world</h1>\n          Hi there! How are <super>you</super> doing?',
            },
          },
        ],
      },
    ],
    embeds.map(i => i.data)
  );
});
