config 	    = require './config'
Root 		= require './dom/root'

api 		= require './api'
auth 		= require './auth'
detect 		= require './detect'

event 		= require './utils/event'
whenReady   = require './utils/ready'

Composition = require './UI/composition'
Editor      = require './UI/editor'
UI          = require './UI'

aye 	= require 'ayepromise'

win 		= window

hasProxy    = false

receivers = {
	'a'	: api.request  # api
	'e'	: api.response # event
	'i'	: Root.connect # init
	'o'	: auth.connect # oauth
	'd' : Composition.apiReady # composition data api ready
	'u' : Composition.connect # composition ready
	'ce': Composition.event # composition event
	'pr': Composition.publicApiCallReply # composition public api reply
	'w' : Editor.connect # editor ready
	'ee': Editor.event # editor event
	'p' : UI.popup.receiver
	'r' : auth.retry
	'v' : UI.plugin.connect # plugin ready
	've': UI.plugin.event # plugin event
	'su': Composition.updateSize
	'startPlaying': Composition.pausePlayingEmbed
	'widgetPublicMethodsAndEvents': Composition.publicMethodsAndEvents
}

# remove the protocol
originRegex = "#{config.lo.replace /(http|https)\:/, ''}|#{config.domain.replace /(http|https)\:/, ''}"
# escape the dots
originRegex = originRegex.replace(/\./g, '\\.')
originRegex = new RegExp(originRegex)

window.widgeticReceiver = receiver = (e) ->
	# console.log("SDK window.widgeticReceiver:", e)
	return unless originRegex.test e.origin
	d = e.data
	try
		return unless typeof d is "string"
		d = JSON.parse(d)
	catch error
		# console.warn 'Widgetic SDK: error parsing JSON:', d
		return

	try
		receivers[d.t]?(d, e)
	catch error
		# commented this error for "TypeError: e.stopPropagation is not a function"
		# console.error 'Widgetic SDK: ', error.stack

Widgetic = ->
	win['WidgeticAsyncInit']?()
	event.on win,'message',receiver
	detect win.location.href
	Root.style()
	setTimeout UI.parse

proxyInitialized = aye.defer()
# TODO: move this inside Root
initProxy = ->
	if !hasProxy
		create = =>
			(@root = new Root()).createProxy()
				.then(-> proxyInitialized.resolve())
				.fail((e) -> proxyInitialized.reject(e))
			hasProxy = true
		if document.getElementsByTagName('body')[0]
			create()
		else
			whenReady create
	return proxyInitialized.promise

Widgetic.prototype.init = (client_id, redirect_uri) ->
	initProxy.call(@)
	return @ unless (client_id and redirect_uri)
	auth.setAuthOptions client_id,redirect_uri
	@

Widgetic.prototype.api = ->
	args = arguments
	return initProxy.call(@)
		.then(-> return api.apply @, args)

Widgetic.prototype.auth = -> auth.apply @, arguments
Widgetic.prototype.auth.register = -> auth.register.apply @, arguments
Widgetic.prototype.auth.status = -> api.getStatus.apply @, arguments
Widgetic.prototype.auth.token  = -> api.accessToken.apply @, arguments
Widgetic.prototype.auth.disconnect  = -> api.disconnect.apply @, arguments

#accessible from outside
Widgetic.prototype.JSON  	= JSON
Widgetic.prototype.Queue 	= api.queue
Widgetic.prototype.Aye		= require 'ayepromise'
Widgetic.prototype.Event 	= event
Widgetic.prototype.GUID 	= require './utils/guid'
Widgetic.prototype.pubsub   = require 'pubsub.js'
Widgetic.prototype.UI       = UI
Widgetic.prototype.find     = require('./UI/embeds').find
Widgetic.prototype.EVENTS   = require './constants/events'
Widgetic.prototype.version  = 'VERSION'
Widgetic.prototype.parse    = detect.parse
Widgetic.prototype.uxhr 	= require 'uxhr'
Widgetic.prototype.debug    = {
	timestamp: -> console.warn('Widgetic.debug.timestamp is deprecated.')
}

module.exports = Widgetic
